# Generated by Django 3.2.7 on 2022-08-01 08:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('purchase', '0008_auto_20220801_0606'),
    ]

    operations = [
        migrations.AlterField(
            model_name='purchase',
            name='bundle_short_version',
            field=models.CharField(max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='bundle_short_version',
            field=models.CharField(max_length=255, null=True),
        ),
    ]
