# Generated by Django 3.2.7 on 2022-07-27 08:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('purchase', '0003_auto_20220215_0611'),
    ]

    operations = [
        migrations.CreateModel(
            name='Subscription',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('user', models.UUIDField()),
                ('currency', models.CharField(max_length=25)),
                ('log_time', models.CharField(max_length=11)),
                ('ext_info', models.CharField(max_length=255)),
                ('product_id', models.CharField(max_length=255)),
                ('transaction_id', models.CharField(max_length=255)),
                ('bundle_short_version', models.CharField(max_length=255)),
                ('fb_user_id', models.CharField(max_length=255, null=True)),
                ('advertiser_id', models.CharField(max_length=255, null=True)),
                ('platform', models.CharField(choices=[('ios', 'Ios'), ('android', 'Android')], max_length=8)),
                ('value_to_sum', models.FloatField()),
                ('body', models.JSONField()),
                ('is_sandbox', models.BooleanField(default=False)),
                ('active_till', models.DateTimeField(null=True)),
                ('google_token', models.CharField(help_text='Only for android platform', max_length=8192, null=True)),
                ('_is_active', models.DateField()),
                ('_expired', models.BooleanField(default=False)),
            ],
        ),
    ]
