# Generated by Django 3.2.7 on 2022-02-15 06:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('purchase', '0002_auto_20210920_1019'),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name='google',
            name='unique',
        ),
        migrations.RemoveField(
            model_name='adjust',
            name='game',
        ),
        migrations.RemoveField(
            model_name='appsflyer',
            name='game',
        ),
        migrations.RemoveField(
            model_name='config',
            name='game',
        ),
        migrations.RemoveField(
            model_name='facebook',
            name='game',
        ),
        migrations.RemoveField(
            model_name='google',
            name='game',
        ),
        migrations.RemoveField(
            model_name='log',
            name='game',
        ),
        migrations.RemoveField(
            model_name='purchase',
            name='game',
        ),
        migrations.AlterField(
            model_name='adjust',
            name='platform',
            field=models.CharField(choices=[('ios', 'Ios'), ('android', 'Android')], max_length=20, unique=True),
        ),
        migrations.AlterField(
            model_name='appsflyer',
            name='platform',
            field=models.CharField(choices=[('ios', 'Ios'), ('android', 'Android')], max_length=20, unique=True),
        ),
        migrations.AlterField(
            model_name='config',
            name='platform',
            field=models.CharField(choices=[('ios', 'Ios'), ('android', 'Android')], max_length=20, unique=True),
        ),
        migrations.AlterField(
            model_name='facebook',
            name='platform',
            field=models.CharField(choices=[('ios', 'Ios'), ('android', 'Android')], max_length=20, unique=True),
        ),
    ]
